import sys
import os.path

def root_length(a, b):
    if b.startswith(a):
        return len(a)
    else:
        return 0

def find_package(path):
    # find out in which package this file is located
    syspaths = sys.path
    syspaths.sort(key=lambda syspath: root_length(syspath, path),
                  reverse=True)
        
    path = path[len(syspaths[0]):]

    # convert path to dotted filename
    if path.startswith(os.path.sep):
        path = path[1:]

    return path
    
class GlobalTemplateManager(object):
    def __init__(self):
        self.templates = {}
        self.paths = {}
        
    def registerDirectory(self, directory):
        for filename in os.listdir(directory):
            if filename.endswith('.pt'):
                self.paths[filename] = "%s/%s" % (directory, filename)

    def unregisterDirectory(self, directory):
        for filename in os.listdir(directory):
            if filename in self.paths:
                del self.paths[filename]
        
    def registerTemplate(self, template):
        # only register templates that have a filename attribute
        if not hasattr(template, 'filename'):
            return
        
        # assert that template is not already registered
        filename = self.templates.get(template)
        if self.paths.get(filename) == template.filename:
            return

        if filename and filename not in self.paths:
            # template file has been unregistered; restore
            # original template
            template.filename = template._filename
            delattr(template, '_filename')
                        
        path = find_package(template.filename)
        filename = path.replace(os.path.sep, '.')

        if filename in self.paths:
            path = self.paths[filename]
            
            # save original filename
            template._filename = template.filename

            # save template and registry and assign path
            template.filename = path
            self.templates[template] = filename

        template._v_last_read = False
        
GlobalTemplateManager = GlobalTemplateManager()

def getGlobalTemplateManager():
    return GlobalTemplateManager



from zope.interface import Interface
from zope.configuration import fields

from Products.CMFPlone import PloneMessageFactory as _

class ITemplateOverridesDirective(Interface):
    """Directive which registers a directory with template overrides."""

    directory = fields.Path(title=_(u"Path to directory"),
                            required=True)




import zope.testing
import unittest

OPTIONFLAGS = (zope.testing.doctest.REPORT_ONLY_FIRST_FAILURE |
               zope.testing.doctest.ELLIPSIS |
               zope.testing.doctest.NORMALIZE_WHITESPACE)

import zope.component.testing

def test_suite():
    doctests = ['README.txt']

    try:
        import Products.Five
        doctests.append('Five.txt')
    except:
        pass
    
    return unittest.TestSuite((
        zope.testing.doctest.DocFileSuite(doctest,
                                          optionflags=OPTIONFLAGS,
                                          setUp=zope.component.testing.setUp,
                                          tearDown=zope.component.testing.tearDown,
                                          package="z3c.jbot") for doctest in doctests
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



import manager
from zope.pagetemplate.pagetemplatefile import PageTemplate

# add registration hook to ``zope.app.pagetemplate``
def jbot(pt_render):
    def render(self, *args, **kwargs): 
        manager.getGlobalTemplateManager().registerTemplate(self)
        return pt_render(self, *args, **kwargs)        
    return render

PageTemplate.pt_render = jbot(PageTemplate.pt_render)



from setuptools import setup, find_packages
import sys, os

version = '0.1.1'

setup(name='z3c.jbot',
      version=version,
      description="Drop-in template overrides.",
      long_description=open("README.txt").read(),
      classifiers=[
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='page template override',
      author='Malthe Borch',
      author_email='mborch@gmail.com',
      url='',
      license='ZPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['z3c'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
